@@ -19,12 +19,12 @@
 #include "boot/stage3b.h"
 #include "utils/buffer.h"
 
-Buffer *stage3a_getblob(const gchar *filename, gsize *loader_size,
-			gsize data_size, GError **err);
-gint build_stage3a(Buffer *dc, gsize dc_size, const Buffer *hdr,
+PvBuffer *stage3a_getblob(const gchar *filename, gsize *loader_size,
+			  gsize data_size, GError **err);
+gint build_stage3a(PvBuffer *dc, gsize dc_size, const PvBuffer *hdr,
 		   struct ipl_parameter_block *ipib, GError **err);
-Buffer *stage3b_getblob(const gchar *filename, GError **err);
-void build_stage3b(Buffer *stage3b, const struct stage3b_args *args);
+PvBuffer *stage3b_getblob(const gchar *filename, GError **err);
+void build_stage3b(PvBuffer *stage3b, const struct stage3b_args *args);
 void memblob_init(struct memblob *arg, uint64_t src, uint64_t size);
 
 #endif
