@@ -31,6 +31,7 @@
 
 #include "buffer.h"
 #include "curl.h"
+#include "openssl_compat.h"
 #include "crypto.h"
 
 #define DEFINE_GSLIST_MAP(t2, t1)					\
@@ -89,15 +90,15 @@ EVP_MD_CTX *digest_ctx_new(const EVP_MD *md, GError **err)
 	return g_steal_pointer(&ctx);
 }
 
-Buffer *digest_ctx_finalize(EVP_MD_CTX *ctx, GError **err)
+PvBuffer *digest_ctx_finalize(EVP_MD_CTX *ctx, GError **err)
 {
 	gint md_size = EVP_MD_size(EVP_MD_CTX_md(ctx));
-	g_autoptr(Buffer) ret = NULL;
+	g_autoptr(PvBuffer) ret = NULL;
 	guint digest_size;
 
 	g_assert(md_size > 0);
 
-	ret = buffer_alloc((guint)md_size);
+	ret = pv_buffer_alloc((guint)md_size);
 	if (EVP_DigestFinal_ex(ctx, ret->data, &digest_size) != 1) {
 		g_set_error(err, PV_CRYPTO_ERROR, PV_CRYPTO_ERROR_INTERNAL,
 			    _("EVP_DigestFinal_ex failed"));
@@ -110,10 +111,10 @@ Buffer *digest_ctx_finalize(EVP_MD_CTX *ctx, GError **err)
 }
 
 /* Returns the digest of @buf using the hash algorithm @md */
-static Buffer *digest_buffer(const EVP_MD *md, const Buffer *buf, GError **err)
+static PvBuffer *digest_buffer(const EVP_MD *md, const PvBuffer *buf, GError **err)
 {
 	g_autoptr(EVP_MD_CTX) md_ctx = NULL;
-	g_autoptr(Buffer) ret = NULL;
+	g_autoptr(PvBuffer) ret = NULL;
 	g_assert(buf);
 
 	md_ctx = digest_ctx_new(md, err);
@@ -134,9 +135,9 @@ static Buffer *digest_buffer(const EVP_MD *md, const Buffer *buf, GError **err)
 }
 
 /* Returns the SHA256 digest of @buf */
-Buffer *sha256_buffer(const Buffer *buf, GError **err)
+PvBuffer *sha256_buffer(const PvBuffer *buf, GError **err)
 {
-	g_autoptr(Buffer) ret = NULL;
+	g_autoptr(PvBuffer) ret = NULL;
 
 	ret = digest_buffer(EVP_sha256(), buf, err);
 	if (!ret)
@@ -207,10 +208,10 @@ union ecdh_pub_key *evp_pkey_to_ecdh_pub_key(EVP_PKEY *key, GError **err)
 	return g_steal_pointer(&ret);
 }
 
-static Buffer *derive_key(EVP_PKEY *cust, EVP_PKEY *host, GError **err)
+static PvBuffer *derive_key(EVP_PKEY *cust, EVP_PKEY *host, GError **err)
 {
 	g_autoptr(EVP_PKEY_CTX) ctx = NULL;
-	g_autoptr(Buffer) ret = NULL;
+	g_autoptr(PvBuffer) ret = NULL;
 	gsize key_size;
 
 	ctx = EVP_PKEY_CTX_new(cust, NULL);
@@ -236,7 +237,7 @@ static Buffer *derive_key(EVP_PKEY *cust, EVP_PKEY *host, GError **err)
 		return NULL;
 	}
 
-	ret = buffer_alloc(key_size);
+	ret = pv_buffer_alloc(key_size);
 	if (EVP_PKEY_derive(ctx, ret->data, &key_size) != 1) {
 		g_set_error(err, PV_CRYPTO_ERROR, PV_CRYPTO_ERROR_DERIVE,
 			    _("Key derivation failed"));
@@ -247,11 +248,11 @@ static Buffer *derive_key(EVP_PKEY *cust, EVP_PKEY *host, GError **err)
 	return g_steal_pointer(&ret);
 }
 
-Buffer *compute_exchange_key(EVP_PKEY *cust, EVP_PKEY *host, GError **err)
+PvBuffer *compute_exchange_key(EVP_PKEY *cust, EVP_PKEY *host, GError **err)
 {
-	g_autoptr(Buffer) raw = buffer_alloc(70);
-	g_autoptr(Buffer) ret = NULL;
-	g_autoptr(Buffer) key = NULL;
+	g_autoptr(PvBuffer) raw = pv_buffer_alloc(70);
+	g_autoptr(PvBuffer) ret = NULL;
+	g_autoptr(PvBuffer) key = NULL;
 	guchar *data;
 
 	key = derive_key(cust, host, err);
@@ -290,10 +291,10 @@ gint generate_tweak(union tweak *tweak, uint16_t i, GError **err)
 	return 0;
 }
 
-static Buffer *generate_rand_data(guint size, const gchar *err_msg,
-				  GError **err)
+static PvBuffer *generate_rand_data(guint size, const gchar *err_msg,
+				    GError **err)
 {
-	g_autoptr(Buffer) buf = buffer_alloc(size);
+	g_autoptr(PvBuffer) buf = pv_buffer_alloc(size);
 
 	g_assert(size <= INT_MAX);
 
@@ -307,14 +308,14 @@ static Buffer *generate_rand_data(guint size, const gchar *err_msg,
 	return g_steal_pointer(&buf);
 }
 
-Buffer *generate_aes_iv(guint size, GError **err)
+PvBuffer *generate_aes_iv(guint size, GError **err)
 {
 	return generate_rand_data(size,
 				  _("Generating a IV failed because the required amount of random data is not available"),
 				  err);
 }
 
-Buffer *generate_aes_key(guint size, GError **err)
+PvBuffer *generate_aes_key(guint size, GError **err)
 {
 	return generate_rand_data(size,
 				  _("Generating a key failed because the required amount of random data is not available"),
@@ -439,11 +440,15 @@ static int check_signature_algo_match(const EVP_PKEY *pkey, const X509 *subject,
 
 static X509_CRL *load_crl_from_bio(BIO *bio)
 {
-	g_autoptr(X509_CRL) crl = PEM_read_bio_X509_CRL(bio, NULL, 0, NULL);
+	g_autoptr(X509_CRL) crl = PEM_read_bio_X509_CRL(bio, NULL, NULL, NULL);
+	gint rc;
+
 	if (crl)
 		return g_steal_pointer(&crl);
 	ERR_clear_error();
-	BIO_reset(bio);
+	rc = BIO_reset(bio);
+	if (rc != 1 || (rc != 0 && BIO_method_type(bio) == BIO_TYPE_FILE))
+		return NULL;
 
 	/* maybe the CRL is stored in DER format */
 	crl = d2i_X509_CRL_bio(bio, NULL);
@@ -514,6 +519,7 @@ X509 *load_cert_from_file(const char *path, GError **err)
 {
 	g_autoptr(BIO) bio = bio_read_from_file(path);
 	g_autoptr(X509) cert = NULL;
+	gint rc;
 
 	if (!bio) {
 		g_set_error(err, PV_CRYPTO_ERROR,
@@ -526,7 +532,12 @@ X509 *load_cert_from_file(const char *path, GError **err)
 	if (cert)
 		return g_steal_pointer(&cert);
 	ERR_clear_error();
-	BIO_reset(bio);
+	rc = BIO_reset(bio);
+	if (rc != 1 || (rc != 0 && BIO_method_type(bio) == BIO_TYPE_FILE)) {
+		g_set_error(err, PV_CRYPTO_ERROR, PV_CRYPTO_ERROR_READ_CERTIFICATE,
+			    _("unable to load certificate: '%s'"), path);
+		return NULL;
+	}
 
 	/* maybe the certificate is stored in DER format */
 	cert = d2i_X509_bio(bio, NULL);
@@ -1428,7 +1439,7 @@ static const char *get_first_dp_url(DIST_POINT *dp)
 	return NULL;
 }
 
-static gboolean insert_crl(X509_NAME *name, X509_CRL *crl)
+static gboolean insert_crl(const X509_NAME *name, X509_CRL *crl)
 {
 	g_autofree gchar *key = NULL;
 
@@ -1443,7 +1454,7 @@ static gboolean insert_crl(X509_NAME *name, X509_CRL *crl)
 }
 
 /* Caller is responsible for free'ing */
-static X509_CRL *lookup_crl(X509_NAME *name)
+static X509_CRL *lookup_crl(const X509_NAME *name)
 {
 	g_autoptr(X509_CRL) crl = NULL;
 	g_autofree gchar *key = NULL;
@@ -1463,7 +1474,7 @@ static X509_CRL *lookup_crl(X509_NAME *name)
 }
 
 /* Returns empty stack if no CRL downloaded. */
-static STACK_OF_X509_CRL *crls_download_cb(X509_STORE_CTX *ctx, X509_NAME *nm)
+static STACK_OF_X509_CRL *crls_download_cb(const X509_STORE_CTX *ctx, const X509_NAME *nm)
 {
 	g_autoptr(STACK_OF_X509_CRL) crls = NULL;
 	g_autoptr(X509_CRL) crl = NULL;
@@ -1473,7 +1484,7 @@ static STACK_OF_X509_CRL *crls_download_cb(X509_STORE_CTX *ctx, X509_NAME *nm)
 	crls = sk_X509_CRL_new_null();
 	if (!crls)
 		g_abort();
-	cert = X509_STORE_CTX_get_current_cert(ctx);
+	cert = Pv_X509_STORE_CTX_get_current_cert(ctx);
 	if (!cert)
 		return g_steal_pointer(&crls);
 	g_assert(X509_NAME_cmp(X509_get_issuer_name(cert), nm) == 0);
@@ -1517,19 +1528,19 @@ void STACK_OF_X509_CRL_free(STACK_OF_X509_CRL *stack)
 /* Downloaded CRLs have a higher precedence than the CRLs specified on the
  * command line.
  */
-static STACK_OF_X509_CRL *crls_cb(X509_STORE_CTX *ctx, X509_NAME *nm)
+static STACK_OF_X509_CRL *crls_cb(const X509_STORE_CTX *ctx, const X509_NAME *nm)
 {
 	g_autoptr(STACK_OF_X509_CRL) crls = crls_download_cb(ctx, nm);
 
 	if (sk_X509_CRL_num(crls) > 0)
 		return g_steal_pointer(&crls);
-	return X509_STORE_CTX_get1_crls(ctx, nm);
+	return Pv_X509_STORE_CTX_get1_crls(ctx, nm);
 }
 
 /* Set up CRL lookup with download support */
 void store_setup_crl_download(X509_STORE *st)
 {
-	X509_STORE_set_lookup_crls(st, crls_cb);
+	Pv_X509_STORE_set_lookup_crls(st, crls_cb);
 }
 
 /* Download a CRL using the URI specified in the distribution @crldp */
@@ -1645,8 +1656,8 @@ gint verify_host_key(X509 *host_key, GSList *issuer_pairs,
 	}
 
 	if (!(verify_flags & X509_V_FLAG_NO_CHECK_TIME)) {
-		const ASN1_TIME *last = X509_get_notBefore(host_key);
-		const ASN1_TIME *next = X509_get_notAfter(host_key);
+		const ASN1_TIME *last = X509_get0_notBefore(host_key);
+		const ASN1_TIME *next = X509_get0_notAfter(host_key);
 
 		if (!last || !next || check_validity_period(last, next)) {
 			g_set_error(err, PV_CRYPTO_ERROR,
@@ -1756,8 +1767,8 @@ static gint __encrypt_decrypt_bio(const struct cipher_parms *parms, BIO *b_in,
 	gint cipher_block_size = EVP_CIPHER_block_size(cipher);
 	guchar in_buf[PAGE_SIZE],
 		out_buf[PAGE_SIZE + (guint)cipher_block_size];
-	const Buffer *key = parms->key;
-	const Buffer *tweak = parms->iv_or_tweak;
+	const PvBuffer *key = parms->key;
+	const PvBuffer *tweak = parms->iv_or_tweak;
 	g_autofree guchar *tmp_tweak = NULL;
 	gint out_len, tweak_size;
 	gsize tmp_size_in = 0, tmp_size_out = 0;
@@ -1847,6 +1858,7 @@ static gint __encrypt_decrypt_bio(const struct cipher_parms *parms, BIO *b_in,
 			g_set_error(err, PV_CRYPTO_ERROR,
 				    PV_CRYPTO_ERROR_INTERNAL,
 				    _("BN_add_word failed"));
+			return -1;
 		}
 		g_assert(BN_num_bytes(tweak_num) > 0);
 		g_assert(BN_num_bytes(tweak_num) <= tweak_size);
@@ -1855,6 +1867,7 @@ static gint __encrypt_decrypt_bio(const struct cipher_parms *parms, BIO *b_in,
 			g_set_error(err, PV_CRYPTO_ERROR,
 				    PV_CRYPTO_ERROR_INTERNAL,
 				    _("BN_bn2binpad failed"));
+			return -1;
 		};
 
 		/* set new tweak */
@@ -1895,11 +1908,11 @@ static gint __encrypt_decrypt_bio(const struct cipher_parms *parms, BIO *b_in,
 	return 0;
 }
 
-static Buffer *__encrypt_decrypt_buffer(const struct cipher_parms *parms,
-					const Buffer *in, gboolean encrypt,
-					GError **err)
+static PvBuffer *__encrypt_decrypt_buffer(const struct cipher_parms *parms,
+					  const PvBuffer *in, gboolean encrypt,
+					  GError **err)
 {
-	g_autoptr(Buffer) ret = NULL;
+	g_autoptr(PvBuffer) ret = NULL;
 	g_autoptr(BIO) b_out = NULL;
 	g_autoptr(BIO) b_in = NULL;
 	gsize in_size, out_size;
@@ -1927,19 +1940,19 @@ static Buffer *__encrypt_decrypt_buffer(const struct cipher_parms *parms,
 		return NULL;
 	}
 
-	ret = buffer_alloc((unsigned long)data_size);
+	ret = pv_buffer_alloc((unsigned long)data_size);
 	memcpy(ret->data, data, ret->size);
 	return g_steal_pointer(&ret);
 }
 
-Buffer *encrypt_buf(const struct cipher_parms *parms, const Buffer *in,
-		    GError **err)
+PvBuffer *encrypt_buf(const struct cipher_parms *parms, const PvBuffer *in,
+		      GError **err)
 {
 	return __encrypt_decrypt_buffer(parms, in, TRUE, err);
 }
 
-Buffer *decrypt_buf(const struct cipher_parms *parms, const Buffer *in,
-		    GError **err)
+PvBuffer *decrypt_buf(const struct cipher_parms *parms, const PvBuffer *in,
+		      GError **err)
 {
 	return __encrypt_decrypt_buffer(parms, in, FALSE, err);
 }
@@ -1993,16 +2006,16 @@ G_GNUC_UNUSED static gint decrypt_file(const struct cipher_parms *parms,
 }
 
 /* GCM mode uses (zero-)padding */
-static int64_t gcm_encrypt_decrypt(const Buffer *in, const Buffer *aad,
+static int64_t gcm_encrypt_decrypt(const PvBuffer *in, const PvBuffer *aad,
 				   const struct cipher_parms *parms,
-				   Buffer *out, Buffer *tag,
+				   PvBuffer *out, PvBuffer *tag,
 				   enum PvCryptoMode mode, GError **err)
 {
 	g_autoptr(EVP_CIPHER_CTX) ctx = NULL;
 	const EVP_CIPHER *cipher = parms->cipher;
-	const Buffer *iv = parms->iv_or_tweak;
+	const PvBuffer *iv = parms->iv_or_tweak;
 	gboolean encrypt = mode == PV_ENCRYPT;
-	const Buffer *key = parms->key;
+	const PvBuffer *key = parms->key;
 	int64_t ret = -1;
 	gint len = -1;
 
@@ -2097,8 +2110,8 @@ static int64_t gcm_encrypt_decrypt(const Buffer *in, const Buffer *aad,
 	return ret;
 }
 
-int64_t gcm_encrypt(const Buffer *in, const Buffer *aad,
-		    const struct cipher_parms *parms, Buffer *out, Buffer *tag,
+int64_t gcm_encrypt(const PvBuffer *in, const PvBuffer *aad,
+		    const struct cipher_parms *parms, PvBuffer *out, PvBuffer *tag,
 		    GError **err)
 {
 	return gcm_encrypt_decrypt(in, aad, parms, out, tag, PV_ENCRYPT, err);
