@@ -9,12 +9,19 @@
  * it under the terms of the MIT license. See LICENSE for details.
  */
 
-#include <time.h>
+#include "lib/util_log.h"
+
 #include "zgetdump.h"
+#include "dfi_mem_chunk.h"
+#include "dfi_vmcoreinfo.h"
+#include "dfi.h"
 
 #define TIME_FMT_STR "%a, %d %b %Y %H:%M:%S %z"
 #define PROGRESS_HASH_CNT 50
 
+#define KDUMP_OLDMEM_BASE	0x10418
+#define KDUMP_OLDMEM_SIZE	0x10420
+
 /*
  * DFI vector - ensure that tape is the first in the list and devmem the second!
  */
@@ -47,17 +54,6 @@ struct cpus {
 	unsigned int		cnt;
 };
 
-/*
- * Memory information
- */
-struct mem {
-	struct dfi_mem_chunk	*chunk_cache;
-	u64			start_addr;
-	u64			end_addr;
-	unsigned int		chunk_cnt;
-	struct util_list	chunk_list;
-};
-
 /*
  * Dump header attribute information
  */
@@ -81,8 +77,6 @@ struct attr {
 static struct {
 	enum dfi_arch	arch;
 	struct attr	attr;
-	struct mem	mem_phys;
-	struct mem	mem_virt;
 	struct cpus	cpus;
 	struct dfi	*dfi;
 	unsigned long	kdump_base;
@@ -109,125 +103,6 @@ static void date_print(void)
 	}
 }
 
-/*
- * Initialize DFI memory chunks
- */
-static void mem_init(struct mem *mem)
-{
-	mem->start_addr = U64_MAX;
-	mem->end_addr = 0;
-	util_list_init(&mem->chunk_list, struct dfi_mem_chunk, list);
-}
-
-/*
- * Memory chunk compare function for list sorting
- */
-static int mem_chunk_cmp_fn(void *a, void *b, void *UNUSED(data))
-{
-	struct dfi_mem_chunk *mem_chunk1 = a;
-	struct dfi_mem_chunk *mem_chunk2 = b;
-
-	return mem_chunk1->start < mem_chunk2->start ? -1 : 1;
-}
-
-/*
- * Update DFI memory chunks
- */
-static void mem_update(struct mem *mem)
-{
-	struct dfi_mem_chunk *mem_chunk;
-
-	util_list_sort(&mem->chunk_list, mem_chunk_cmp_fn, NULL);
-	mem->start_addr = U64_MAX;
-	mem->end_addr = 0;
-	util_list_iterate(&mem->chunk_list, mem_chunk) {
-		mem->start_addr = MIN(mem->start_addr, mem_chunk->start);
-		mem->end_addr = MAX(mem->end_addr, mem_chunk->end);
-	}
-}
-
-/*
- * Print memory map
- */
-static void mem_map_print(void)
-{
-	struct dfi_mem_chunk *mem_chunk;
-	u64 print_start = 0, print_end = 0;
-	const char *zero_str;
-	u32 volnr = 0;
-
-	STDERR("\nMemory map:\n");
-	/*
-	 * Print each memory chunk if verbose specified
-	 */
-	if (g.opts.verbose_specified) {
-		dfi_mem_chunk_iterate(mem_chunk) {
-			zero_str = "";
-			if (mem_chunk->read_fn == dfi_mem_chunk_read_zero)
-				zero_str = " zeroes";
-			STDERR("  %016llx - %016llx (%llu MB%s)\n",
-			       mem_chunk->start, mem_chunk->end,
-			       TO_MIB(mem_chunk->size), zero_str);
-		}
-		return;
-	}
-	/*
-	 * Merge adjacent memory chunks from the same volume
-	 */
-	dfi_mem_chunk_iterate(mem_chunk) {
-		if (print_end == 0) {
-			print_start = mem_chunk->start;
-			print_end = mem_chunk->end;
-			volnr = mem_chunk->volnr;
-			continue;
-		}
-		if (mem_chunk->start != print_end + 1 ||
-		    mem_chunk->volnr != volnr) {
-			STDERR("  %016llx - %016llx (%llu MB)\n", print_start,
-			       print_end, TO_MIB(print_end - print_start + 1));
-			print_start = mem_chunk->start;
-			volnr = mem_chunk->volnr;
-		}
-		print_end = mem_chunk->end;
-	}
-	STDERR("  %016llx - %016llx (%llu MB)\n", print_start,
-	       print_end, TO_MIB(print_end - print_start + 1));
-}
-
-/*
- * Is memory range valid?
- */
-int dfi_mem_range_valid(u64 addr, u64 len)
-{
-	struct dfi_mem_chunk *mem_chunk;
-	u64 addr_end = addr + len;
-
-	do {
-		mem_chunk = dfi_mem_chunk_find(addr);
-		if (!mem_chunk)
-			return 0;
-		addr += MIN(len, mem_chunk->end - addr + 1);
-	} while (addr < addr_end);
-	return 1;
-}
-
-/*
- * Is memory already mapped at range?
- */
-static int mem_range_mapped(u64 start, u64 size)
-{
-	struct dfi_mem_chunk *mem_chunk;
-	u64 end = start + size - 1;
-
-	dfi_mem_chunk_iterate(mem_chunk) {
-		if (mem_chunk->start > end)
-			continue;
-		if (mem_chunk->end < start)
-			continue;
-		return 1;
-	}
-	return 0;
-}
 /*
  * Print dump information (--info option)
  */
@@ -236,7 +111,7 @@ void dfi_info_print(void)
 	STDERR("General dump info:\n");
 	STDERR("  Dump format........: %s\n", l.dfi->name);
 	if (l.attr.dfi_version)
-		STDERR("  Version............: %d\n", *l.attr.dfi_version);
+		STDERR("  Version............: %u\n", *l.attr.dfi_version);
 	date_print();
 	if (l.attr.dump_method)
 		STDERR("  Dump method........: %s\n", l.attr.dump_method);
@@ -248,259 +123,32 @@ void dfi_info_print(void)
 		STDERR("  UTS kernel version.: %s\n", l.attr.utsname->version);
 	}
 	if (l.attr.vol_nr)
-		STDERR("  Volume number......: %d\n", *l.attr.vol_nr);
+		STDERR("  Volume number......: %u\n", *l.attr.vol_nr);
 	if (l.attr.build_arch)
 		STDERR("  Build arch.........: %s\n",
-		      dfi_arch_str(*l.attr.build_arch));
+		       dfi_arch_str(*l.attr.build_arch));
 	STDERR("  System arch........: %s\n", dfi_arch_str(l.arch));
 	if (l.cpus.cnt)
-		STDERR("  CPU count (online).: %d\n", l.cpus.cnt);
+		STDERR("  CPU count (online).: %u\n", l.cpus.cnt);
 	if (l.attr.real_cpu_cnt)
-		STDERR("  CPU count (real)...: %d\n", *l.attr.real_cpu_cnt);
+		STDERR("  CPU count (real)...: %u\n", *l.attr.real_cpu_cnt);
 	if (dfi_mem_range())
 		STDERR("  Dump memory range..: %lld MB\n",
 		       TO_MIB(dfi_mem_range()));
 	if (l.attr.mem_size_real)
 		STDERR("  Real memory range..: %lld MB\n",
-		      TO_MIB(*l.attr.mem_size_real));
+		       TO_MIB(*l.attr.mem_size_real));
 	if (l.attr.file_size)
 		STDERR("  Dump file size.....: %lld MB\n",
-		      TO_MIB(*l.attr.file_size));
+		       TO_MIB(*l.attr.file_size));
 	if (dfi_mem_range())
-		mem_map_print();
+		dfi_mem_map_print(g.opts.verbose);
 	if (l.dfi->info_dump) {
 		STDERR("\nDump device info:\n");
 		l.dfi->info_dump();
 	}
 }
 
-/*
- * Add memory chunk to memory
- */
-static void mem_chunk_create(struct mem *mem, u64 start, u64 size, void *data,
-			     dfi_mem_chunk_read_fn read_fn,
-			     dfi_mem_chunk_free_fn free_fn)
-{
-	struct dfi_mem_chunk *mem_chunk;
-
-	mem_chunk = zg_alloc(sizeof(*mem_chunk));
-	mem_chunk->start = start;
-	mem_chunk->end = start + size - 1;
-	mem_chunk->size = size;
-	mem_chunk->read_fn = read_fn;
-	mem_chunk->free_fn = free_fn;
-	mem_chunk->data = data;
-
-	util_list_add_tail(&mem->chunk_list, mem_chunk);
-	mem->start_addr = MIN(mem->start_addr, mem_chunk->start);
-	mem->end_addr = MAX(mem->end_addr, mem_chunk->end);
-	mem->chunk_cache = mem_chunk;
-	mem->chunk_cnt++;
-}
-
-/*
- * Check if memory chunk contains address
- */
-static int mem_chunk_has_addr(struct dfi_mem_chunk *mem_chunk, u64 addr)
-{
-	return (addr >= mem_chunk->start && addr <= mem_chunk->end);
-}
-
-/*
- * Find memory chunk that contains address
- */
-static struct dfi_mem_chunk *mem_chunk_find(struct mem *mem, u64 addr)
-{
-	struct dfi_mem_chunk *mem_chunk;
-
-	if (mem_chunk_has_addr(mem->chunk_cache, addr))
-		return mem->chunk_cache;
-	util_list_iterate(&mem->chunk_list, mem_chunk) {
-		if (mem_chunk_has_addr(mem_chunk, addr)) {
-			mem->chunk_cache = mem_chunk;
-			return mem_chunk;
-		}
-	}
-	return NULL;
-}
-
-/*
- * Read memory at given address
- */
-static void mem_read(struct mem *mem, u64 addr, void *buf, size_t cnt)
-{
-	struct dfi_mem_chunk *mem_chunk;
-	u64 size, off, copied = 0;
-
-	while (copied != cnt) {
-		mem_chunk = mem_chunk_find(mem, addr);
-		size = MIN(cnt - copied, mem_chunk->end - addr + 1);
-		off = addr - mem_chunk->start;
-		mem_chunk->read_fn(mem_chunk, off, buf + copied, size);
-		copied += size;
-		addr += size;
-	}
-}
-
-/*
- * Read memory for virtual map memory chunk
- */
-static void mem_chunk_map_read_fn(struct dfi_mem_chunk *mem_chunk, u64 off,
-				  void *buf, u64 cnt)
-{
-	u64 *start = mem_chunk->data;
-
-	dfi_mem_phys_read(*start + off, buf, cnt);
-}
-
-/*
- * Check if memory chunk is a virtual mapping
- */
-static int mem_chunk_is_map(struct dfi_mem_chunk *mem_chunk)
-{
-	return mem_chunk->read_fn == mem_chunk_map_read_fn;
-}
-
-/*
- * Return physical start address for memory chunk
- */
-static u64 mem_chunk_start_phys(struct dfi_mem_chunk *mem_chunk)
-{
-	if (mem_chunk_is_map(mem_chunk))
-		return *((u64 *) mem_chunk->data);
-	else
-		return mem_chunk->start;
-}
-
-/*
- * Add virtual memory chunk with simple virtual mapping
- */
-static void mem_chunk_map_add(u64 start, u64 size, u64 start_p)
-{
-	u64 *data = zg_alloc(sizeof(*data));
-
-	*data = start_p;
-	dfi_mem_chunk_virt_add(start, size, data, mem_chunk_map_read_fn,
-			       zg_free);
-}
-
-/*
- * Add virtual memory chunk
- */
-void dfi_mem_chunk_virt_add(u64 start, u64 size, void *data,
-			    dfi_mem_chunk_read_fn read_fn,
-			    dfi_mem_chunk_free_fn free_fn)
-{
-	if (size == 0)
-		return;
-	mem_chunk_create(&l.mem_virt, start, size, data, read_fn, free_fn);
-}
-
-/*
- * Add memory chunk with volume index
- */
-void dfi_mem_chunk_add_vol(u64 start, u64 size, void *data,
-			   dfi_mem_chunk_read_fn read_fn,
-			   dfi_mem_chunk_free_fn free_fn,
-			   u32 volnr)
-{
-	if (size == 0)
-		return;
-	mem_chunk_create(&l.mem_phys, start, size, data, read_fn, free_fn);
-	mem_chunk_create(&l.mem_virt, start, size, data, read_fn, NULL);
-	l.mem_virt.chunk_cache->volnr = volnr;
-
-}
-
-/*
- * Add memory chunk
- */
-void dfi_mem_chunk_add(u64 start, u64 size, void *data,
-		       dfi_mem_chunk_read_fn read_fn,
-		       dfi_mem_chunk_free_fn free_fn)
-{
-	dfi_mem_chunk_add_vol(start, size, data, read_fn, free_fn, 0);
-}
-
-/*
- * Read zero pages
- */
-void dfi_mem_chunk_read_zero(struct dfi_mem_chunk *UNUSED(mem_chunk),
-			     u64 UNUSED(off), void *buf, u64 cnt)
-{
-	memset(buf, 0, cnt);
-}
-
-/*
- * Return mem_chunk list head
- */
-struct util_list *dfi_mem_chunk_list(void)
-{
-	return &l.mem_virt.chunk_list;
-}
-
-/*
- * Return number of memory chunks in input dump
- */
-unsigned int dfi_mem_chunk_cnt(void)
-{
-	return l.mem_virt.chunk_cnt;
-}
-
-/*
- * Return maximum memory range
- */
-u64 dfi_mem_range(void)
-{
-	if (l.mem_virt.start_addr == U64_MAX)
-		return 0;
-	return l.mem_virt.end_addr - l.mem_virt.start_addr + 1;
-}
-
-/*
- * Return first memory chunk
- */
-struct dfi_mem_chunk *dfi_mem_chunk_first(void)
-{
-	if (util_list_is_empty(&l.mem_virt.chunk_list))
-		return NULL;
-	return util_list_start(&l.mem_virt.chunk_list);
-}
-
-/*
- * Return last memory chunk
- */
-struct dfi_mem_chunk *dfi_mem_chunk_last(void)
-{
-	if (util_list_is_empty(&l.mem_virt.chunk_list))
-		return NULL;
-	return util_list_end(&l.mem_virt.chunk_list);
-}
-
-/*
- * Return next memory chunk
- */
-struct dfi_mem_chunk *dfi_mem_chunk_next(struct dfi_mem_chunk *mem_chunk)
-{
-	return util_list_next(&l.mem_virt.chunk_list, mem_chunk);
-}
-
-/*
- * Return previous memory chunk
- */
-struct dfi_mem_chunk *dfi_mem_chunk_prev(struct dfi_mem_chunk *mem_chunk)
-{
-	return util_list_prev(&l.mem_virt.chunk_list, mem_chunk);
-}
-
-/*
- * Find memory chunk for given address
- */
-struct dfi_mem_chunk *dfi_mem_chunk_find(u64 addr)
-{
-	return mem_chunk_find(&l.mem_virt, addr);
-}
-
 /*
  * Initialize CPU info
  */
@@ -600,33 +248,6 @@ struct util_list *dfi_cpu_list(void)
 	return &l.cpus.list;
 }
 
-/*
- * Read memory at given address and do kdump swap if necessary
- */
-void dfi_mem_read(u64 addr, void *buf, size_t cnt)
-{
-	mem_read(&l.mem_virt, addr, buf, cnt);
-}
-
-/*
- * Read physical memory at given address
- */
-void dfi_mem_phys_read(u64 addr, void *buf, size_t cnt)
-{
-	mem_read(&l.mem_phys, addr, buf, cnt);
-}
-
-/*
- * Read memory at given address with return code
- */
-int dfi_mem_read_rc(u64 addr, void *buf, size_t cnt)
-{
-	if (!dfi_mem_range_valid(addr, cnt))
-		return -EINVAL;
-	dfi_mem_read(addr, buf, cnt);
-	return 0;
-}
-
 /*
  * Get input dump format name
  */
@@ -665,6 +286,7 @@ const char *dfi_arch_str(enum dfi_arch arch)
 		return "unknown";
 	}
 	ABORT("dfi_arch_str: Invalid dfi arch: %d", arch);
+	return NULL; /* UNREACHABLE */
 }
 
 /*
@@ -886,7 +508,7 @@ static void lc2cpu_64(struct dfi_cpu *cpu, struct dfi_lowcore_64 *lc)
 	/* Add VX registers if available */
 	if (!dfi_cpu_lc_has_vx_sa(lc))
 		return;
-	if (dfi_mem_read_rc(lc->vector_save_area_addr, &vx_sa, sizeof(vx_sa))) {
+	if (dfi_mem_virt_read(lc->vector_save_area_addr, &vx_sa, sizeof(vx_sa))) {
 		STDERR("zgetdump: Vector registers save area is beyond dump memory limit for CPU %d\n", cpu->cpu_id);
 		return;
 	}
@@ -917,7 +539,7 @@ static void lc2cpu_32(struct dfi_cpu_32 *cpu, struct dfi_lowcore_32 *lc)
  * Note: When this function is called, the memory chunks have to be already
  *       defined by the DFI dump specific code.
  */
-void dfi_cpu_add_from_lc(u32 lc_addr)
+int dfi_cpu_add_from_lc(u32 lc_addr)
 {
 	struct dfi_cpu *cpu = dfi_cpu_alloc();
 
@@ -930,12 +552,14 @@ void dfi_cpu_add_from_lc(u32 lc_addr)
 		if (l.arch == DFI_ARCH_32) {
 			struct dfi_cpu_32 cpu_32;
 			struct dfi_lowcore_32 lc;
-			dfi_mem_read(lc_addr, &lc, sizeof(lc));
+			if (dfi_mem_virt_read(lc_addr, &lc, sizeof(lc)))
+				return -EINVAL;
 			lc2cpu_32(&cpu_32, &lc);
 			cpu_32_to_64(cpu, &cpu_32);
 		} else {
 			struct dfi_lowcore_64 lc;
-			dfi_mem_read(lc_addr, &lc, sizeof(lc));
+			if (dfi_mem_virt_read(lc_addr, &lc, sizeof(lc)))
+				return -EINVAL;
 			lc2cpu_64(cpu, &lc);
 		}
 		break;
@@ -943,6 +567,7 @@ void dfi_cpu_add_from_lc(u32 lc_addr)
 		ABORT("dfi_cpu_add_from_lc() called for CONTENT_NONE");
 	}
 	dfi_cpu_add(cpu);
+	return 0;
 }
 
 /*
@@ -984,89 +609,6 @@ unsigned long dfi_kdump_base(void)
 	return l.kdump_base;
 }
 
-/*
- * Unmap memory region
- */
-static void mem_unmap(u64 start, u64 size)
-{
-	u64 start_phys, end_phys, addr_phys, addr_virt, size_virt;
-	struct dfi_mem_chunk *mem_chunk, *tmp;
-	u64 end = start + size - 1;
-
-	util_list_iterate_safe(&l.mem_virt.chunk_list, mem_chunk, tmp) {
-		/*
-		 * Chunk not hit?
-		 */
-		if (mem_chunk->start >= start + size)
-			continue;
-		if (mem_chunk->end < start)
-			continue;
-		/*
-		 * Chunk completely unmapped
-		 *
-		 * UNMAP: UUUUUUUUU || UUUUUU
-		 * CHUNK:   CCCC    || CCCCCC
-		 * TO:
-		 */
-		if (mem_chunk->start >= start && mem_chunk->end <= end)
-			goto free;
-
-		/*
-		 * Get real start and end addresses
-		 */
-		start_phys = mem_chunk_start_phys(mem_chunk);
-		end_phys = start_phys + mem_chunk->size - 1;
-
-		/*
-		 * Chunk hit at start or in the middle?
-		 *
-		 * UNMAP: UUUUUU   ||   UU    || UUU
-		 * CHUNK:    CCCCC || CCCCCC  ||   CCCC
-		 * TO:          NN ||     NN  ||    NNN
-		 */
-		if (mem_chunk->end > end) {
-			addr_virt = end + 1;
-			size_virt = mem_chunk->end - end;
-			addr_phys = end_phys - size_virt + 1;
-			mem_chunk_map_add(addr_virt, size_virt, addr_phys);
-		}
-		/*
-		 * Chunk hit at end or in the middle?
-		 *
-		 * UNMAP:   UUUUUU   ||   UU    ||   UUU
-		 * CHUNK: CCCCC      || CCCCCC  || CCC
-		 * TO:    NN         || NN      || NN
-		 */
-		if (mem_chunk->start < start) {
-			addr_virt = mem_chunk->start;
-			size_virt = start - addr_virt;
-			addr_phys = start_phys;
-			mem_chunk_map_add(addr_virt, size_virt, addr_phys);
-		}
-free:
-		util_list_remove(&l.mem_virt.chunk_list, mem_chunk);
-		l.mem_virt.chunk_cnt--;
-		if (mem_chunk->data && mem_chunk->free_fn)
-			mem_chunk->free_fn(mem_chunk->data);
-		zg_free(mem_chunk);
-	}
-	mem_update(&l.mem_virt);
-}
-
-/*
- * Map memory region
- */
-static void mem_map(u64 start, u64 size, u64 start_phys)
-{
-	if (mem_range_mapped(start, size)) {
-		mem_map_print();
-		ABORT("Map request for already mapped region (%llx/%llx/%llx)",
-		      start, size, start_phys);
-	}
-	mem_chunk_map_add(start, size, start_phys);
-	mem_update(&l.mem_virt);
-}
-
 /*
  * Check if dump contains a kdump dump and initialize kdump_base and kdump_size
  */
@@ -1074,8 +616,12 @@ static void kdump_init(void)
 {
 	unsigned long base, size;
 
-	dfi_mem_phys_read(0x10418, &base, sizeof(base));
-	dfi_mem_phys_read(0x10420, &size, sizeof(size));
+	util_log_print(UTIL_LOG_TRACE, "DFI kdump initialization\n");
+
+	if (dfi_mem_phys_read(KDUMP_OLDMEM_BASE, &base, sizeof(base)))
+		return;
+	if (dfi_mem_phys_read(KDUMP_OLDMEM_SIZE, &size, sizeof(size)))
+		return;
 	if (base == 0 || size == 0)
 		return;
 	if (base % MIB || size % MIB)
@@ -1084,6 +630,9 @@ static void kdump_init(void)
 		return;
 	l.kdump_base = base;
 	l.kdump_size = size;
+	util_log_print(UTIL_LOG_INFO,
+		       "DFI found valid kdump base 0x%016lx size 0x%016lx\n",
+		       l.kdump_base, l.kdump_size);
 	/*
 	 * For dumped kdump and user has selected "prod" we swap
 	 * the crashkernel memory with old memory. If user selected "kdump",
@@ -1093,11 +642,11 @@ static void kdump_init(void)
 	if (!g.opts.select_specified)
 		return;
 	if (g.opts.select == OPTS_SELECT_PROD) {
-		mem_unmap(0, size);
-		mem_unmap(base, size);
-		mem_map(0, size, base);
+		dfi_mem_unmap(0, size);
+		dfi_mem_unmap(base, size);
+		dfi_mem_map(0, size, base);
 	} else if (g.opts.select == OPTS_SELECT_KDUMP) {
-		mem_unmap(l.kdump_size, U64_MAX - l.kdump_size);
+		dfi_mem_unmap(l.kdump_size, U64_MAX - l.kdump_size);
 	}
 }
 
@@ -1126,8 +675,8 @@ static void kdump_select_prod_init(void)
 	}
 	dfi_cpu_info_init(DFI_CPU_CONTENT_ALL);
 	for (i = 0; i < count; i++) {
-		if (dfi_mem_read_rc(ptr + i * sizeof(long), &prefix,
-				   sizeof(prefix)))
+		if (dfi_mem_virt_read(ptr + i * sizeof(long), &prefix,
+				      sizeof(prefix)))
 			continue;
 		if (prefix == 0)
 			continue;
@@ -1146,9 +695,11 @@ static void utsname_init(void)
 	unsigned long ptr;
 	char buf[1024];
 
+	util_log_print(UTIL_LOG_TRACE, "DFI utsname initialization\n");
+
 	if (dfi_vmcoreinfo_symbol(&ptr, "init_uts_ns"))
 		return;
-	if (dfi_mem_read_rc(ptr, buf, sizeof(buf)))
+	if (dfi_mem_virt_read(ptr, buf, sizeof(buf)))
 		return;
 	utsname = memchr(buf, 'L', sizeof(buf) - sizeof(*utsname));
 	if (!utsname)
@@ -1156,6 +707,8 @@ static void utsname_init(void)
 	if (strncmp(utsname->sysname, "Linux", sizeof(utsname->version)) != 0)
 		return;
 	dfi_attr_utsname_set(utsname);
+	util_log_print(UTIL_LOG_INFO, "DFI utsname release %s version %s\n",
+		       utsname->release, utsname->version);
 }
 
 /*
@@ -1165,7 +718,9 @@ static void livedump_init(void)
 {
 	u64 magic;
 
-	if (dfi_mem_read_rc(0, &magic, sizeof(magic)))
+	util_log_print(UTIL_LOG_TRACE, "DFI livedump initialization\n");
+
+	if (dfi_mem_virt_read(0, &magic, sizeof(magic)))
 		return;
 	if (magic == dfi_live_dump_magic)
 		dfi_attr_dump_method_set(DFI_DUMP_METHOD_LIVE);
@@ -1201,12 +756,16 @@ int dfi_init(void)
 	struct dfi *dfi;
 	int i = 0, rc;
 
+	util_log_print(UTIL_LOG_TRACE, "DFI initialization\n");
+
 	l.arch = DFI_ARCH_UNKNOWN;
-	mem_init(&l.mem_virt);
-	mem_init(&l.mem_phys);
+	rc = dfi_mem_chunk_init();
+	if (rc)
+		return rc;
 	attr_init();
 	dfi_cpu_info_init(DFI_CPU_CONTENT_NONE);
 	while ((dfi = dfi_vec[i])) {
+		util_log_print(UTIL_LOG_DEBUG, "DFI trying %s\n", dfi->name);
 		l.dfi = dfi;
 		g.fh = dfi_dump_open(g.opts.device);
 		rc = dfi->init();
@@ -1218,12 +777,15 @@ int dfi_init(void)
 			utsname_init();
 			livedump_init();
 		}
+		util_log_print(UTIL_LOG_DEBUG, "DFI %s returned with rc %d\n",
+			       dfi->name, rc);
 		if (rc == 0 || rc == -EINVAL)
 			return rc;
 		zg_close(g.fh);
 		i++;
 	}
 	ERR_EXIT("No valid dump found on \"%s\"", g.opts.device);
+	return -1; /* UNREACHABLE */
 }
 
 /*
@@ -1231,6 +793,8 @@ int dfi_init(void)
  */
 void dfi_exit(void)
 {
+	util_log_print(UTIL_LOG_TRACE, "DFI exit\n");
+
 	if (l.dfi && l.dfi->exit)
 		l.dfi->exit();
 }
