@@ -54,11 +54,14 @@ char *strcat(char *, const char *);
 int strncmp(const char *, const char *, unsigned long);
 int strlen(const char *);
 char *strcpy(char *, const char *);
+size_t strlcpy(char *dest, const char *src, size_t size);
+char *strchr(const char *str, char c);
+unsigned int strhash(unsigned char *str, unsigned int hash_size);
 unsigned long get_zeroed_page(void);
 void free_page(unsigned long);
 void initialize(void);
 void libc_stop(unsigned long) __noreturn;
-void start(void);
+void __noreturn start(void);
 void pgm_check_handler(void);
 void pgm_check_handler_fn(void);
 void panic_notify(unsigned long reason);
