@@ -143,12 +143,13 @@ int menu(void)
 	int rc;
 
 	cmd_line_extra = (char *)COMMAND_LINE_EXTRA;
+	memset(cmd_line_extra, 0, COMMAND_LINE_EXTRA_SIZE);
+
 	rc = sclp_setup(SCLP_INIT);
 	if (rc)
 		/* sclp setup failed boot default */
 		goto boot;
 
-	memset(cmd_line_extra, 0, COMMAND_LINE_SIZE);
 	rc = menu_param(&value);
 	if (rc == 0) {
 		/* got number from loadparm, boot it */
@@ -187,7 +188,7 @@ boot:
 
 	/* append 'BOOT_IMAGE=<num>' to parmline */
 	snprintf(endstring, sizeof(endstring), " BOOT_IMAGE=%u", value);
-	if ((strlen(cmd_line_extra) + strlen(endstring)) < COMMAND_LINE_SIZE)
+	if ((strlen(cmd_line_extra) + strlen(endstring)) < LEGACY_COMMAND_LINE_SIZE)
 		strcat(cmd_line_extra, endstring);
 
 	sclp_setup(SCLP_DISABLE);
