@@ -39,29 +39,34 @@ struct job_target_data {
 	blocknum_t targetoffset;
 };
 
-struct job_ipl_data {
+struct job_common_ipl_data {
 	char* image;
 	char* parmline;
 	char* ramdisk;
 	address_t image_addr;
 	address_t parm_addr;
 	address_t ramdisk_addr;
+};
+
+struct job_ipl_data {
+	struct job_common_ipl_data common;
+	address_t envblk_addr;
 	int is_kdump;
 };
 
+struct job_envblk_data {
+	int size;
+	char *buf;
+};
+
 struct job_segment_data {
 	char* segment;
 	address_t segment_addr;
 };
 
 struct job_dump_data {
+	struct job_common_ipl_data common;
 	char* device;
-	char* image;
-	char* parmline;
-	char* ramdisk;
-	address_t image_addr;
-	address_t parm_addr;
-	address_t ramdisk_addr;
 	uint64_t mem;
 };
 
@@ -74,13 +79,8 @@ struct job_mvdump_data {
 };
 
 struct job_ipl_tape_data {
+	struct job_common_ipl_data common;
 	char* device;
-	char* image;
-	char* parmline;
-	char* ramdisk;
-	address_t image_addr;
-	address_t parm_addr;
-	address_t ramdisk_addr;
 };
 
 
@@ -108,6 +108,7 @@ struct job_menu_data {
 struct job_data {
 	enum job_id id;
 	struct job_target_data target;
+	struct job_envblk_data envblk;
 	char* name;
 	union {
 		struct job_ipl_data ipl;
