@@ -14,6 +14,7 @@
 
 #include <stdint.h>
 #include "lib/zt_common.h"
+#include "lib/util_base.h"
 #include "boot/loaders_layout.h"
 
 #define ZIPL_MAGIC			"zIPL"
@@ -22,7 +23,8 @@
 
 #define ADDRESS_LIMIT			0x80000000UL
 #define UNSPECIFIED_ADDRESS		-1UL
-#define MAXIMUM_PARMLINE_SIZE		0x380UL
+#define LEGACY_MAXIMUM_PARMLINE_SIZE	0x380UL
+#define MAXIMUM_PARMLINE_SIZE		0x10000UL
 #define MAXIMUM_PHYSICAL_BLOCKSIZE	0x1000UL
 
 #define BOOTMAP_FILENAME		"bootmap"
@@ -35,7 +37,7 @@
 #define ZIPL_DEFAULT_CONF		TOOLS_SYSCONFDIR "/zipl.conf"
 #define ZIPL_MINIMAL_CONF		TOOLS_LIBDIR "/zipl.conf"
 #define ZIPL_DEFAULT_BLSDIR		"/boot/loader/entries"
-#define ZIPL_STAGE3_PATH		TOOLS_LIBDIR "/stage3.bin"
+#define ZIPL_STAGE3_PATH		util_libdir_path("stage3.bin")
 #define ZIPL_SIPL_PATH			"/sys/firmware/ipl/has_secure"
 
 #define MENU_DEFAULT_PROMPT		0
