@@ -17,6 +17,7 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <assert.h>
 
 #include "lib/zt_common.h"
 #include "lib/util_part.h"
@@ -26,18 +27,14 @@
 
 #include "boot.h"
 #include "bootmap.h"
+#include "envblk.h"
 #include "disk.h"
 #include "error.h"
 #include "install.h"
 #include "misc.h"
 
-/* Header text of the bootmap file */
-static const char header_text[] = "zSeries bootmap file\n"
-				  "created by zIPL\n";
-
 /* Pointer to dedicated empty block in bootmap. */
-disk_blockptr_t empty_block;
-
+static disk_blockptr_t empty_block;
 
 /* Get size of a bootmap block pointer for disk with given INFO. */
 static int
@@ -145,7 +142,7 @@ check_secure_boot_support(void)
  * of segment table blocks to the file identified by file descriptor FD. Upon
  * success, return 0 and set SECTION_POINTER to point to the first block in
  * the resulting segment table. Return non-zero otherwise. */
-int
+static int
 add_segment_table(int fd, disk_blockptr_t* list, blocknum_t count,
 		  disk_blockptr_t* segment_pointer,
 		  struct disk_info* info)
@@ -291,10 +288,11 @@ struct component_loc {
 };
 
 static int
-add_component_file(int fd, const char* filename, address_t load_address,
-		   size_t trailer, void *component, int add_files,
-		   struct disk_info* info, struct job_target_data* target,
-		   struct component_loc *location)
+add_component_file_range(int fd, const char *filename, struct file_range *reg,
+			 address_t load_address,
+			 size_t trailer, void *component, int add_files,
+			 struct disk_info *info, struct job_target_data *target,
+			 struct component_loc *location)
 {
 	struct disk_info* file_info;
 	struct component_loc loc;
@@ -306,6 +304,7 @@ add_component_file(int fd, const char* filename, address_t load_address,
 	int rc;
 
 	if (add_files) {
+		assert(reg == NULL); /* not implemented */
 		/* Read file to buffer */
 		rc = misc_read_file(filename, &buffer, &size, 0);
 		if (rc) {
@@ -332,8 +331,8 @@ add_component_file(int fd, const char* filename, address_t load_address,
 			return -1;
 		}
 		/* Get block list from existing file */
-		count = disk_get_blocklist_from_file(filename, &list,
-						     file_info);
+		count = disk_get_blocklist_from_file(filename, reg,
+						     &list, file_info);
 		disk_free_info(file_info);
 		if (count == 0)
 			return -1;
@@ -358,9 +357,22 @@ add_component_file(int fd, const char* filename, address_t load_address,
 }
 
 static int
-add_component_buffer(int fd, void* buffer, size_t size, component_data data,
-		     void* component, struct disk_info* info,
-		     struct component_loc *location, int type)
+add_component_file(int fd, const char *filename, address_t load_address,
+		   size_t trailer, void *component, int add_files,
+		   struct disk_info *info, struct job_target_data *target,
+		   struct component_loc *location)
+{
+	return add_component_file_range(fd, filename, NULL, load_address,
+					trailer, component, add_files,
+					info, target, location);
+}
+
+static int
+add_component_buffer_align(int fd, void *buffer, size_t size,
+			   component_data data, void *component,
+			   struct disk_info *info,
+			   struct component_loc *location, int type,
+			   int align, off_t *offset)
 {
 	struct component_loc loc;
 	disk_blockptr_t segment;
@@ -369,7 +381,8 @@ add_component_buffer(int fd, void* buffer, size_t size, component_data data,
 	int rc;
 
 	/* Write buffer */
-	count = disk_write_block_buffer(fd, 0, buffer, size, &list, info);
+	count = disk_write_block_buffer_align(fd, 0, buffer, size, &list, info,
+					      align, offset);
 	if (count == 0) {
 		error_text("Could not write to bootmap file");
 		return -1;
@@ -396,6 +409,15 @@ add_component_buffer(int fd, void* buffer, size_t size, component_data data,
 	return rc;
 }
 
+static int
+add_component_buffer(int fd, void *buffer, size_t size, component_data data,
+		     void *component, struct disk_info *info,
+		     struct component_loc *location, int type)
+{
+	return add_component_buffer_align(fd, buffer, size, data, component,
+					  info, location, type,
+					  info->phy_block_size, NULL);
+}
 
 static int
 add_dummy_buffer(int fd, size_t size, address_t addr, void *component,
@@ -440,7 +462,7 @@ print_components(const char *name[], struct component_loc *loc, int num)
 }
 
 static int
-extract_signature(char *filename, void **ret_signature,
+extract_signature(const char *filename, void **ret_signature,
 		  struct signature_header *sig_head)
 {
 	struct file_signature *file_sig;
@@ -496,8 +518,9 @@ check_remaining_filesize(size_t filesize, size_t signature_size,
 	}
 }
 
-static int
-add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
+static int add_ipl_program(int fd, char *filename,
+		bool add_envblk, struct job_envblk_data *envblk,
+		struct job_ipl_data *ipl, disk_blockptr_t *program,
 		int verbose, int add_files, component_header_type type,
 		struct disk_info* info, struct job_target_data* target,
 		int is_secure)
@@ -507,12 +530,13 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	size_t ramdisk_size, image_size;
 	bool secure_boot_supported;
 	size_t stage3_params_size;
-	const char *comp_name[10];
+	const char *comp_name[11];
 	size_t signature_size;
 	int offset;
 	uint64_t flags = 0;
 	void *stage3_params;
 	struct stat stats;
+	off_t envblk_off;
 	void *signature;
 	int comp_nr = 0;
 	void *table;
@@ -536,16 +560,16 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	 */
 	/* initiate values for ramdisk */
 	stats.st_size = 0;
-	if (ipl->ramdisk != NULL) {
+	if (ipl->common.ramdisk != NULL) {
 		/* Add ramdisk */
 		if (verbose) {
-			printf("  initial ramdisk...: %s\n", ipl->ramdisk);
+			printf("  initial ramdisk...: %s\n", ipl->common.ramdisk);
 		}
 		/* Get ramdisk file size */
-		if (stat(ipl->ramdisk, &stats)) {
+		if (stat(ipl->common.ramdisk, &stats)) {
 			error_reason(strerror(errno));
 			error_text("Could not get information for file '%s'",
-				   ipl->ramdisk);
+				   ipl->common.ramdisk);
 			free(table);
 			return -1;
 		}
@@ -585,10 +609,10 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		flags |= STAGE3_FLAG_KDUMP;
 
 	/* Get kernel file size */
-	if (stat(ipl->image, &stats)) {
+	if (stat(ipl->common.image, &stats)) {
 		error_reason(strerror(errno));
 		error_text("Could not get information for file '%s'",
-			   ipl->image);
+			   ipl->common.image);
 		free(table);
 		return -1;
 	}
@@ -645,12 +669,14 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 
 	/* Add stage 3 parameter to bootmap */
 	rc = boot_get_stage3_parms(&stage3_params, &stage3_params_size,
-				   ipl->parm_addr, ipl->ramdisk_addr,
+				   ipl->common.parm_addr, ipl->common.ramdisk_addr,
 				   ramdisk_size,
 				   ipl->is_kdump ? IMAGE_ENTRY_KDUMP :
 				   IMAGE_ENTRY,
 				   (info->type == disk_type_scsi) ? 0 : 1,
-				   flags, ipl->image_addr, image_size);
+				   flags, ipl->common.image_addr, image_size,
+				   ipl->envblk_addr,
+				   add_envblk ? envblk->size : 0);
 	if (rc) {
 		free(table);
 		return rc;
@@ -672,14 +698,14 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 
 	/* Add kernel image */
 	if (verbose) {
-		printf("  kernel image......: %s\n", ipl->image);
+		printf("  kernel image......: %s\n", ipl->common.image);
 	}
-	signature_size = extract_signature(ipl->image, &signature, &sig_head);
+	signature_size = extract_signature(ipl->common.image, &signature, &sig_head);
 	if (signature_size &&
 	    (is_secure == SECURE_BOOT_ENABLED ||
 	     (is_secure == SECURE_BOOT_AUTO && secure_boot_supported))) {
 		if (verbose)
-			printf("  signature for.....: %s\n", ipl->image);
+			printf("  signature for.....: %s\n", ipl->common.image);
 
 		rc = add_component_buffer(fd, signature, sig_head.length,
 					  (component_data)sig_head,
@@ -696,7 +722,7 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		comp_nr++;
 		free(signature);
 		check_remaining_filesize(image_size, signature_size, info,
-					 ipl->image);
+					 ipl->common.image);
 	} else if (is_secure == SECURE_BOOT_ENABLED) {
 		/*
 		 * If secure boot is forced and we have failed to extract a
@@ -705,16 +731,16 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		 */
 		error_text("Could not install Secure Boot IPL records");
 		error_reason("Missing signature in image file %s",
-			     ipl->image);
+			     ipl->common.image);
 		free(table);
 		return -1;
 	}
 
-	rc = add_component_file(fd, ipl->image, ipl->image_addr,
+	rc = add_component_file(fd, ipl->common.image, ipl->common.image_addr,
 				signature_size, VOID_ADD(table, offset),
 				add_files, info, target, &comp_loc[comp_nr]);
 	if (rc) {
-		error_text("Could not add image file '%s'", ipl->image);
+		error_text("Could not add image file '%s'", ipl->common.image);
 		free(table);
 		return rc;
 	}
@@ -723,19 +749,19 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	comp_nr++;
 
 	/* Add kernel parmline */
-	if (ipl->parmline != NULL) {
+	if (ipl->common.parmline != NULL) {
 		if (verbose) {
-			printf("  kernel parmline...: '%s'\n", ipl->parmline);
+			printf("  kernel parmline...: '%s'\n", ipl->common.parmline);
 		}
-		rc = add_component_buffer(fd, ipl->parmline,
-					  strlen(ipl->parmline) + 1,
-					  (component_data) ipl->parm_addr,
+		rc = add_component_buffer(fd, ipl->common.parmline,
+					  strlen(ipl->common.parmline) + 1,
+					  (component_data) ipl->common.parm_addr,
 					  VOID_ADD(table, offset),
 					  info, &comp_loc[comp_nr],
 					  component_load);
 		if (rc) {
 			error_text("Could not add parmline '%s'",
-				   ipl->parmline);
+				   ipl->common.parmline);
 			free(table);
 			return -1;
 		}
@@ -743,10 +769,9 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		comp_name[comp_nr] = "parmline";
 		comp_nr++;
 	}
-
-	/* finally add ramdisk */
-	if (ipl->ramdisk != NULL) {
-		signature_size = extract_signature(ipl->ramdisk, &signature,
+	/* add ramdisk */
+	if (ipl->common.ramdisk != NULL) {
+		signature_size = extract_signature(ipl->common.ramdisk, &signature,
 						   &sig_head);
 		if (signature_size &&
 		    (is_secure == SECURE_BOOT_ENABLED ||
@@ -754,7 +779,7 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		      secure_boot_supported))) {
 			if (verbose) {
 				printf("  signature for.....: %s\n",
-				       ipl->ramdisk);
+				       ipl->common.ramdisk);
 			}
 			rc = add_component_buffer(fd, signature,
 						  sig_head.length,
@@ -772,16 +797,16 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 			comp_nr++;
 			free(signature);
 			check_remaining_filesize(ramdisk_size, signature_size,
-						 info, ipl->ramdisk);
+						 info, ipl->common.ramdisk);
 		}
-		rc = add_component_file(fd, ipl->ramdisk,
-					ipl->ramdisk_addr, signature_size,
+		rc = add_component_file(fd, ipl->common.ramdisk,
+					ipl->common.ramdisk_addr, signature_size,
 					VOID_ADD(table, offset),
 					add_files, info, target,
 					&comp_loc[comp_nr]);
 		if (rc) {
 			error_text("Could not add ramdisk '%s'",
-				   ipl->ramdisk);
+				   ipl->common.ramdisk);
 			free(table);
 			return -1;
 		}
@@ -789,6 +814,65 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		comp_name[comp_nr] = "initial ramdisk";
 		comp_nr++;
 	}
+	if (add_envblk == true) {
+		/*
+		 * finally add environment block
+		 */
+		rc = envblk_offset_get(fd, &envblk_off);
+		if (rc) {
+			free(table);
+			return rc;
+		}
+		if (envblk_off == 0) {
+			/*
+			 * write with fs_block_size alignment to make sure that the
+			 * logical environment block will get to single file system
+			 * block
+			 */
+			rc = add_component_buffer_align(fd,
+					       envblk->buf, envblk->size,
+					       (component_data)ipl->envblk_addr,
+					       VOID_ADD(table, offset),
+					       info, &comp_loc[comp_nr],
+					       component_load,
+					       info->fs_block_size,
+					       &envblk_off);
+			if (rc) {
+				error_text("Could not add environment block");
+				free(table);
+				return rc;
+			}
+			assert(envblk_off % info->fs_block_size == 0);
+			/*
+			 * store environment block location
+			 * in the bootmap header
+			 */
+			rc = envblk_offset_set(fd, envblk_off);
+			if (rc) {
+				error_text("Could not store environment block location");
+				free(table);
+				return rc;
+			}
+		} else {
+			struct file_range reg;
+
+			reg.offset = envblk_off;
+			reg.len = envblk->size;
+			rc = add_component_file_range(fd, filename, &reg,
+						      ipl->envblk_addr, 0,
+						      VOID_ADD(table, offset),
+						      0, info, target,
+						      &comp_loc[comp_nr]);
+			if (rc) {
+				error_text("Could not add environment block");
+				free(table);
+				return rc;
+			}
+		}
+		offset += sizeof(struct component_entry);
+		comp_name[comp_nr] = "environment blk";
+		comp_nr++;
+	}
 	if (verbose)
 		print_components(comp_name, comp_loc, comp_nr);
 	/* Terminate component table */
@@ -834,7 +918,7 @@ add_segment_program(int fd, struct job_segment_data* segment,
 	rc = add_component_file(fd, segment->segment, segment->segment_addr, 0,
 				VOID_ADD(table, offset), add_files, info,
 				target, &comp_loc[0]);
-if (rc) {
+	if (rc) {
 		error_text("Could not add segment file '%s'",
 			   segment->segment);
 		free(table);
@@ -925,18 +1009,15 @@ add_dump_program(int fd, struct job_dump_data* dump,
 
 	/* Convert fs dump job to IPL job */
 	memset(&ipl, 0, sizeof(ipl));
-	ipl.image = dump->image;
-	ipl.image_addr = dump->image_addr;
-	ipl.ramdisk = dump->ramdisk;
-	ipl.ramdisk_addr = dump->ramdisk_addr;
+	ipl.common = dump->common;
 
 	/* Get file system dump parmline */
-	rc = get_dump_parmline(dump->device, dump->parmline,
-			       info, target, &ipl.parmline);
+	rc = get_dump_parmline(dump->device, dump->common.parmline,
+			       info, target, &ipl.common.parmline);
 	if (rc)
 		return rc;
-	ipl.parm_addr = dump->parm_addr;
-	return add_ipl_program(fd, &ipl, program, verbose, 1,
+	ipl.common.parm_addr = dump->common.parm_addr;
+	return add_ipl_program(fd, NULL, false, NULL, &ipl, program, verbose, 1,
 			       type, info, target, SECURE_BOOT_DISABLED);
 }
 
@@ -944,8 +1025,8 @@ add_dump_program(int fd, struct job_dump_data* dump,
 /* Build a program table from job data and set pointer to program table
  * block upon success. */
 static int
-build_program_table(int fd, struct job_data* job, disk_blockptr_t* pointer,
-		    struct disk_info* info)
+build_program_table(int fd, char *filename, struct job_data *job,
+		    disk_blockptr_t *pointer, struct disk_info *info)
 {
 	disk_blockptr_t* table;
 	int entries, component_header;
@@ -959,6 +1040,7 @@ build_program_table(int fd, struct job_data* job, disk_blockptr_t* pointer,
 						entries);
 	if (table == NULL)
 		return -1;
+
 	memset((void *) table, 0, sizeof(disk_blockptr_t) * entries);
 	/* Add programs */
 	switch (job->id) {
@@ -972,8 +1054,9 @@ build_program_table(int fd, struct job_data* job, disk_blockptr_t* pointer,
 			component_header = component_header_dump;
 		else
 			component_header = component_header_ipl;
-		rc = add_ipl_program(fd, &job->data.ipl, &table[0],
-				     verbose || job->command_line,
+		rc = add_ipl_program(fd, filename,
+				     true, &job->envblk, &job->data.ipl,
+				     &table[0], verbose || job->command_line,
 				     job->add_files, component_header,
 				     info, &job->target, job->is_secure);
 		break;
@@ -1026,7 +1109,8 @@ build_program_table(int fd, struct job_data* job, disk_blockptr_t* pointer,
 				else
 					is_secure =
 					      job->data.menu.entry[i].is_secure;
-				rc = add_ipl_program(fd,
+				rc = add_ipl_program(fd, filename,
+					true, &job->envblk,
 					&job->data.menu.entry[i].data.ipl,
 					&table[job->data.menu.entry[i].pos],
 					verbose || job->command_line,
@@ -1060,6 +1144,9 @@ build_program_table(int fd, struct job_data* job, disk_blockptr_t* pointer,
 		rc = -1;
 		break;
 	}
+	if (job->envblk.buf && verbose)
+		envblk_print(job->envblk.buf, job->envblk.size);
+
 	if (rc == 0) {
 		/* Add program table block */
 		rc = add_program_table(fd, table, entries, pointer, info);
@@ -1193,14 +1280,14 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 		size = IMAGE_LOAD_ADDRESS;
 
 		/* Ramdisk */
-		if (job->data.dump.ramdisk != NULL) {
-			if (stat(job->data.dump.ramdisk, &st))
+		if (job->data.dump.common.ramdisk != NULL) {
+			if (stat(job->data.dump.common.ramdisk, &st))
 				goto out_misc_free_temp_dev;
 			size += DIV_ROUND_UP(st.st_size, info->phy_block_size);
 			size += 1; /* For ramdisk section entry */
 		}
 		/* Kernel */
-		if (stat(job->data.dump.image, &st))
+		if (stat(job->data.dump.common.image, &st))
 			goto out_misc_free_temp_dev;
 		size += DIV_ROUND_UP(st.st_size - IMAGE_LOAD_ADDRESS,
 				     info->phy_block_size);
@@ -1218,9 +1305,9 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 		scsi_sb.dump_size = unused_size;
 	}
 
-	/* Write bootmap header */
-	if (misc_write(fd, header_text, sizeof(header_text))) {
-		error_text("Could not write to file '%s'", filename);
+	/* Initialize bootmap header */
+	if (bootmap_header_init(fd)) {
+		error_text("Could not init bootmap header at '%s'", filename);
 		goto out_misc_free_temp_dev;
 	}
 	/* Write empty block to be read in place of holes in files */
@@ -1229,7 +1316,7 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 		goto out_misc_free_temp_dev;
 	}
 	/* Build program table */
-	if (build_program_table(fd, job, program_table, info))
+	if (build_program_table(fd, filename, job, program_table, info))
 		goto out_misc_free_temp_dev;
 	if (job->id == job_dump_partition) {
 		scsi_sb.magic = SCSI_DUMP_SB_MAGIC;
