@@ -14,16 +14,21 @@
 #include <getopt.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <fcntl.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <assert.h>
+
+#include "lib/util_arch.h"
 
 #include "error.h"
 #include "job.h"
 #include "misc.h"
 #include "scan.h"
 #include "zipl.h"
+#include "envblk.h"
 
 /* Command line options */
 static struct option options[] = {
@@ -35,6 +40,7 @@ static struct option options[] = {
 	{ "targetgeometry",     required_argument,      NULL, 0xac},
 	{ "targetblocksize",    required_argument,      NULL, 0xad},
 	{ "targetoffset",       required_argument,      NULL, 0xae},
+	{ "environment",        required_argument,      NULL, 0xaf},
 	{ "image", 		required_argument,	NULL, 'i'},
 	{ "ramdisk",		required_argument,	NULL, 'r'},
 	{ "parmfile",		required_argument,	NULL, 'p'},
@@ -71,6 +77,7 @@ static const char *zipl_conf[] = {
 struct command_line {
 	char* data[SCAN_KEYWORD_NUM];
 	char* config;
+	char *envblk_import_hint;
 	char *blsdir;
 	char* menu;
 	char* section;
@@ -85,7 +92,6 @@ struct command_line {
 	enum scan_section_type type;
 };
 
-
 static int
 store_option(struct command_line* cmdline, enum scan_keyword_id keyword,
 	     char* value)
@@ -209,6 +215,13 @@ get_command_line(int argc, char* argv[], struct command_line* line)
 			rc = store_option(&cmdline, scan_keyword_targetoffset,
 					  optarg);
 			break;
+		case 0xaf:
+			if (cmdline.envblk_import_hint != NULL) {
+				error_reason("Option 'environment' specified more than once");
+				rc = -1;
+			} else
+				cmdline.envblk_import_hint = optarg;
+			break;
 		case 'T':
 			is_keyword = 1;
 			rc = store_option(&cmdline, scan_keyword_tape,
@@ -345,55 +358,50 @@ get_command_line(int argc, char* argv[], struct command_line* line)
 static void
 free_target_data(struct job_target_data* data)
 {
-	if (data->targetbase != NULL)
-		free(data->targetbase);
+	free(data->targetbase);
+}
+
+static void
+free_envblk_data(struct job_envblk_data *data)
+{
+	free(data->buf);
+}
+
+static void
+free_common_ipl_data(struct job_common_ipl_data *common)
+{
+	free(common->image);
+	free(common->parmline);
+	free(common->ramdisk);
 }
 
 static void
 free_ipl_data(struct job_ipl_data* data)
 {
-	if (data->image != NULL)
-		free(data->image);
-	if (data->parmline != NULL)
-		free(data->parmline);
-	if (data->ramdisk != NULL)
-		free(data->ramdisk);
+	free_common_ipl_data(&data->common);
 }
 
 
 static void
 free_ipl_tape_data(struct job_ipl_tape_data* data)
 {
-	if (data->device != NULL)
-		free(data->device);
-	if (data->image != NULL)
-		free(data->image);
-	if (data->parmline != NULL)
-		free(data->parmline);
-	if (data->ramdisk != NULL)
-		free(data->ramdisk);
+	free(data->device);
+	free_common_ipl_data(&data->common);
 }
 
 
 static void
 free_segment_data(struct job_segment_data* data)
 {
-	if (data->segment != NULL)
-		free(data->segment);
+	free(data->segment);
 }
 
 
 static void
 free_dump_data(struct job_dump_data* data)
 {
-	if (data->device != NULL)
-		free(data->device);
-	if (data->image != NULL)
-		free(data->image);
-	if (data->parmline != NULL)
-		free(data->parmline);
-	if (data->ramdisk != NULL)
-		free(data->ramdisk);
+	free(data->device);
+	free_common_ipl_data(&data->common);
 }
 
 
@@ -404,8 +412,7 @@ free_menu_data(struct job_menu_data* data)
 
 	if (data->entry != NULL) {
 		for (i=0; i < data->num; i++) {
-			if (data->entry[i].name != NULL)
-				free(data->entry[i].name);
+			free(data->entry[i].name);
 			switch (data->entry[i].id) {
 			case job_ipl:
 				free_ipl_data(&data->entry[i].data.ipl);
@@ -423,27 +430,25 @@ free_menu_data(struct job_menu_data* data)
 }
 
 
-void
+static void
 free_mvdump_data(struct job_mvdump_data* data)
 {
 	int i;
 
-	if (data->device_list != NULL)
-		free(data->device_list);
+	free(data->device_list);
 	for (i = 0; i < data->device_count; i++)
-		if (data->device[i] != NULL)
-			free(data->device[i]);
+		free(data->device[i]);
 }
 
 
 void
 job_free(struct job_data* job)
 {
-	if (job->target.bootmap_dir != NULL)
-		free(job->target.bootmap_dir);
+
+	free(job->target.bootmap_dir);
 	free_target_data(&job->target);
-	if (job->name != NULL)
-		free(job->name);
+	free_envblk_data(&job->envblk);
+	free(job->name);
 	switch (job->id) {
 	case job_ipl:
 		free_ipl_data(&job->data.ipl);
@@ -478,7 +483,7 @@ struct component_loc {
 };
 
 static int
-set_cl_element(struct component_loc *cl, char *name, char *filename,
+set_cl_element(struct component_loc *cl, char *name, const char *filename,
 	       address_t *addrp, size_t size, off_t off, off_t align)
 {
 	struct stat stats;
@@ -491,6 +496,7 @@ set_cl_element(struct component_loc *cl, char *name, char *filename,
 		cl->size = ALIGN(size, MAXIMUM_PHYSICAL_BLOCKSIZE);
 		return 0;
 	}
+	assert(filename != NULL);
 	/* Get size */
 	if (stat(filename, &stats)) {
 		error_reason(strerror(errno));
@@ -527,75 +533,38 @@ sort_cl_array(struct component_loc* cl, int elements)
 	}
 }
 
+
 static int
-get_ipl_components(struct job_ipl_data *ipl, struct component_loc **clp,
-		   int *nump)
+get_common_components(struct job_common_ipl_data *common,
+		      struct component_loc **clp, int *nump,
+		      int extra)
 {
 	struct component_loc *cl;
 	int num;
 	int rc;
 
-	/* Get memory for image, parmline, ramdisk */
-	cl = misc_calloc(3, sizeof(struct component_loc));
+	/*
+	 * Get memory for image, parmline, ramdisk, loader and
+	 * possible extra components
+	 */
+	cl = misc_calloc(3 + extra, sizeof(struct component_loc));
 	if (cl == NULL)
 		return -1;
 	/* Fill in component data */
 	num = 0;
-	rc = set_cl_element(&cl[num++], "kernel image", ipl->image,
-			    &ipl->image_addr, 0, 0,
+	rc = set_cl_element(&cl[num++], "kernel image", common->image,
+			    &common->image_addr, 0, 0,
 			    MAXIMUM_PHYSICAL_BLOCKSIZE);
 	if (rc)
 		goto error;
-	if (ipl->parmline) {
-		rc = set_cl_element(&cl[num++], "parmline", NULL,
-				    &ipl->parm_addr, MAXIMUM_PARMLINE_SIZE, 0,
-				    MAXIMUM_PHYSICAL_BLOCKSIZE);
-		if (rc)
-			goto error;
-	}
-	if (ipl->ramdisk) {
-		rc = set_cl_element(&cl[num++], "initial ramdisk", ipl->ramdisk,
-			&ipl->ramdisk_addr, 0, 0, 0x10000);
-		if (rc)
-			goto error;
-	}
-
-	*clp = cl;
-	*nump = num;
-	return 0;
-error:
-	free(cl);
-	return rc;
-}
-
-
-static int
-get_dump_components(struct job_dump_data *dump,
-		       struct component_loc **clp, int *nump)
-{
-	struct component_loc *cl;
-	int num;
-	int rc;
-
-	/* Get memory for image, parmline, ramdisk, loader */
-	cl = misc_calloc(3, sizeof(struct component_loc));
-	if (cl == NULL)
-		return -1;
-	/* Fill in component data */
-	num = 0;
-	rc = set_cl_element(&cl[num++], "kernel image", dump->image,
-			    &dump->image_addr, 0, 0x0,
-			    MAXIMUM_PHYSICAL_BLOCKSIZE);
-	if (rc)
-		goto error;
-	rc = set_cl_element(&cl[num++], "parmline", NULL, &dump->parm_addr,
+	rc = set_cl_element(&cl[num++], "parmline", NULL, &common->parm_addr,
 			    MAXIMUM_PARMLINE_SIZE, 0,
 			    MAXIMUM_PHYSICAL_BLOCKSIZE);
 	if (rc)
 		goto error;
-	if (dump->ramdisk) {
+	if (common->ramdisk) {
 		rc = set_cl_element(&cl[num++], "initial ramdisk",
-				    dump->ramdisk, &dump->ramdisk_addr,
+				    common->ramdisk, &common->ramdisk_addr,
 				    0, 0, 0x10000);
 		if (rc)
 			goto error;
@@ -611,40 +580,23 @@ error:
 
 
 static int
-get_ipl_tape_components(struct job_ipl_tape_data *ipl_tape,
-			struct component_loc **clp, int *nump)
+get_ipl_components(struct job_ipl_data *ipl, struct component_loc **clp,
+		   int *nump, struct job_envblk_data *envblk)
 {
 	struct component_loc *cl;
 	int num;
 	int rc;
 
-	/* Get memory for image, parmline, ramdisk */
-	cl = misc_calloc(3, sizeof(struct component_loc));
-	if (cl == NULL)
-		return -1;
-	/* Fill in component data */
-	num = 0;
-	rc = set_cl_element(&cl[num++], "kernel image", ipl_tape->image,
-			    &ipl_tape->image_addr, 0, 0x10000,
+	rc = get_common_components(&ipl->common, &cl, &num, 1);
+	if (rc)
+		return rc;
+	rc = set_cl_element(&cl[num++], "environment block",
+			    NULL,
+			    &ipl->envblk_addr, envblk->size, 0,
 			    MAXIMUM_PHYSICAL_BLOCKSIZE);
 	if (rc)
 		goto error;
-	/* Enhance the image size value for tape loader specific logic */
-	cl[0].size = ALIGN(cl[0].size + 0x100, MAXIMUM_PHYSICAL_BLOCKSIZE);
-	if (ipl_tape->parmline) {
-		rc = set_cl_element(&cl[num++], "parmline", NULL,
-				    &ipl_tape->parm_addr, MAXIMUM_PARMLINE_SIZE,
-				    0, MAXIMUM_PHYSICAL_BLOCKSIZE);
-		if (rc)
-			goto error;
-	}
-	if (ipl_tape->ramdisk) {
-		rc = set_cl_element(&cl[num++], "initial ramdisk",
-				    ipl_tape->ramdisk, &ipl_tape->ramdisk_addr,
-				    0, 0, 0x10000);
-		if (rc)
-			goto error;
-	}
+
 	*clp = cl;
 	*nump = num;
 	return 0;
@@ -751,15 +703,17 @@ finalize_component_address_data(struct component_loc *cl, int num,
 
 
 static int
-finalize_ipl_address_data(struct job_ipl_data* ipl, char *name)
+finalize_ipl_address_data(struct job_ipl_data *ipl, char *name,
+			  struct job_envblk_data *envblk)
 {
 	unsigned long address_limit;
 	struct component_loc *cl;
 	int num;
 	int rc;
 
-	address_limit = ipl->is_kdump ? ADDRESS_LIMIT_KDUMP : ADDRESS_LIMIT;
-	rc = get_ipl_components(ipl, &cl, &num);
+	address_limit = ipl->is_kdump ?
+		MIN(util_arch_hsa_maxsize(), ADDRESS_LIMIT) : ADDRESS_LIMIT;
+	rc = get_ipl_components(ipl, &cl, &num, envblk);
 	if (rc)
 		return rc;
 	sort_cl_array(cl, num);
@@ -774,13 +728,13 @@ out_free:
 
 
 static int
-finalize_dump_address_data(struct job_dump_data* dump, char *name)
+finalize_common_address_data(struct job_common_ipl_data *common, char *name)
 {
 	struct component_loc *cl;
 	int num;
 	int rc;
 
-	rc = get_dump_components(dump, &cl, &num);
+	rc = get_common_components(common, &cl, &num, 0);
 	if (rc)
 		return rc;
 	sort_cl_array(cl, num);
@@ -793,67 +747,69 @@ out_free:
 	return rc;
 }
 
-
-static int
-finalize_ipl_tape_address_data(struct job_ipl_tape_data* ipl_tape, char *name)
+static void error_text_section(const char *text, const char *section, const char *file)
 {
-	struct component_loc *cl;
-	int num;
-	int rc;
-	int i;
-
-	rc = get_ipl_tape_components(ipl_tape, &cl, &num);
-	if (rc)
-		return rc;
-	sort_cl_array(cl, num);
-	rc = check_component_address_data(cl, num, name, ADDRESS_LIMIT);
-	if (rc)
-		goto out_free;
-	rc = finalize_component_address_data(cl, num, ADDRESS_LIMIT);
-	/* The loader needs a ramdisk_addr anyway */
-	if (ipl_tape->ramdisk_addr == 0) {
-		for (i = 0; i < num; i++) {
-			if (*cl[i].addrp == ipl_tape->image_addr)
-				ipl_tape->ramdisk_addr =
-					*cl[i].addrp + cl[i].size;
-		}
+	if (section == NULL) {
+		error_text("%s '%s'", text, file);
+	} else {
+		error_text("%s '%s' in section '%s'", text, file, section);
 	}
-out_free:
-	free(cl);
-	return rc;
 }
 
 
 static int
-check_job_ipl_data(struct job_ipl_data *ipl, char* name)
+check_common_ipl_data(struct job_common_ipl_data *common, const char *section)
+{
+	uint64_t max_parm_size, len;
+	char *buffer = NULL;
+	size_t size;
+	int rc;
+
+	if (common->image != NULL) {
+		rc = misc_read_file(common->image, &buffer, &size, 0);
+		if (rc) {
+			error_text_section("Image file", section, common->image);
+			return rc;
+		}
+
+		if (size < MAX_COMMAND_LINE_SIZE + sizeof(uint64_t)) {
+			error_text_section("Image file", section, common->image);
+			return -1;
+		}
+
+		max_parm_size = *(uint64_t *)(buffer + MAX_COMMAND_LINE_SIZE);
+		if (!max_parm_size)
+			max_parm_size = LEGACY_MAXIMUM_PARMLINE_SIZE;
+
+		len = strlen(common->parmline);
+		if (len > max_parm_size) {
+			error_text("The length of the parameters line "
+				   "(%d bytes) exceeds the allowed maximum "
+				   "(%d bytes) in section '%s'", len, max_parm_size, section);
+			return -1;
+		}
+	}
+
+	if (common->ramdisk != NULL) {
+		rc = misc_check_readable_file(common->ramdisk);
+		if (rc) {
+			error_text_section("Ramdisk file", section, common->ramdisk);
+			return rc;
+		}
+	}
+	return 0;
+}
+
+static int
+check_job_ipl_data(struct job_ipl_data *ipl, char *name,
+		   struct job_envblk_data *envblk)
 {
 	int rc;
 
-	if (ipl->image != NULL) {
-		rc = misc_check_readable_file(ipl->image);
-		if (rc) {
-			if (name == NULL) {
-				error_text("Image file '%s'", ipl->image);
-			} else {
-				error_text("Image file '%s' in section '%s'",
-					   ipl->image, name);
-			}
-			return rc;
-		}
-	}
-	if (ipl->ramdisk != NULL) {
-		rc = misc_check_readable_file(ipl->ramdisk);
-		if (rc) {
-			if (name == NULL) {
-				error_text("Ramdisk file '%s'", ipl->ramdisk);
-			} else {
-				error_text("Ramdisk file '%s' in section '%s'",
-					   ipl->ramdisk, name);
-			}
-			return rc;
-		}
-	}
-	return finalize_ipl_address_data(ipl, name);
+	rc = check_common_ipl_data(&ipl->common, name);
+	if (rc)
+		return rc;
+	return finalize_ipl_address_data(ipl, name, envblk);
 }
 
 
@@ -865,13 +821,7 @@ check_job_segment_data(struct job_segment_data* segment, char* name)
 	if (segment->segment != NULL) {
 		rc = misc_check_readable_file(segment->segment);
 		if (rc) {
-			if (name == NULL) {
-				error_text("Segment file '%s'",
-					   segment->segment);
-			} else {
-				error_text("Segment file '%s' in section '%s'",
-					   segment->segment, name);
-			}
+			error_text_section("Segment file", name, segment->segment);
 			return rc;
 		}
 	}
@@ -887,12 +837,7 @@ check_job_dump_data(struct job_dump_data* dump, char* name)
 	if (dump->device != NULL) {
 		rc = misc_check_writable_device(dump->device, 1, 1);
 		if (rc) {
-			if (name == NULL) {
-				error_text("Dump device '%s'", dump->device);
-			} else {
-				error_text("Dump device '%s' in section '%s'",
-					   dump->device, name);
-			}
+			error_text_section("Dump device", name, dump->device);
 			return rc;
 		}
 	}
@@ -911,28 +856,28 @@ check_job_dump_images(struct job_dump_data* dump, char* name)
 			   ZFCPDUMP_IMAGE);
 		return rc;
 	}
-	dump->image = misc_strdup(ZFCPDUMP_IMAGE);
-	if (dump->image == NULL)
+	dump->common.image = misc_strdup(ZFCPDUMP_IMAGE);
+	if (dump->common.image == NULL)
 		return -1;
-	dump->image_addr = IMAGE_LOAD_ADDRESS;
+	dump->common.image_addr = IMAGE_LOAD_ADDRESS;
 
 	/* Ramdisk is no longer required with new initramfs dump system */
 	if (misc_check_readable_file(ZFCPDUMP_INITRD))
-		dump->ramdisk = NULL;
+		dump->common.ramdisk = NULL;
 	else {
-		dump->ramdisk = misc_strdup(ZFCPDUMP_INITRD);
-		if (dump->ramdisk == NULL)
+		dump->common.ramdisk = misc_strdup(ZFCPDUMP_INITRD);
+		if (dump->common.ramdisk == NULL)
 			return -1;
-		dump->ramdisk_addr = UNSPECIFIED_ADDRESS;
+		dump->common.ramdisk_addr = UNSPECIFIED_ADDRESS;
 	}
 
-	dump->parm_addr = UNSPECIFIED_ADDRESS;
-	return finalize_dump_address_data(dump, name);
+	dump->common.parm_addr = UNSPECIFIED_ADDRESS;
+	return finalize_common_address_data(&dump->common, name);
 }
 
 
 static int
-check_job_menu_data(struct job_menu_data* menu)
+check_job_menu_data(struct job_menu_data *menu, struct job_envblk_data *envblk)
 {
 	int rc;
 	int i;
@@ -941,7 +886,8 @@ check_job_menu_data(struct job_menu_data* menu)
 		switch (menu->entry[i].id) {
 		case job_ipl:
 			rc = check_job_ipl_data(&menu->entry[i].data.ipl,
-						menu->entry[i].name);
+						menu->entry[i].name,
+						envblk);
 			if (rc)
 				return rc;
 			break;
@@ -967,40 +913,14 @@ check_job_ipl_tape_data(struct job_ipl_tape_data *ipl, char* name)
 	if (ipl->device != NULL) {
 		rc = misc_check_writable_device(ipl->device, 1, 1);
 		if (rc) {
-			if (name == NULL) {
-				error_text("Tape device '%s'", ipl->device);
-			} else {
-				error_text("Tape device '%s' in section '%s'",
-					   ipl->device, name);
-			}
+			error_text_section("Tape device", name, ipl->device);
 			return rc;
 		}
 	}
-	if (ipl->image != NULL) {
-		rc = misc_check_readable_file(ipl->image);
-		if (rc) {
-			if (name == NULL) {
-				error_text("Image file '%s'", ipl->image);
-			} else {
-				error_text("Image file '%s' in section '%s'",
-					   ipl->image, name);
-			}
-			return rc;
-		}
-	}
-	if (ipl->ramdisk != NULL) {
-		rc = misc_check_readable_file(ipl->ramdisk);
-		if (rc) {
-			if (name == NULL) {
-				error_text("Ramdisk file '%s'", ipl->ramdisk);
-			} else {
-				error_text("Ramdisk file '%s' in section '%s'",
-					   ipl->ramdisk, name);
-			}
-			return rc;
-		}
-	}
-	return finalize_ipl_tape_address_data(ipl, name);
+	rc = check_common_ipl_data(&ipl->common, name);
+	if (rc)
+		return rc;
+	return finalize_common_address_data(&ipl->common, name);
 }
 
 static int
@@ -1012,12 +932,7 @@ check_job_mvdump_data(struct job_mvdump_data* dump, char* name)
 
 	rc = misc_read_file(dump->device_list, &buffer, &size, 0);
 	if (rc) {
-		if (name == NULL) {
-			error_text("Dump target list '%s'", dump->device_list);
-		} else {
-			error_text("Dump target list '%s' in section '%s'",
-				   dump->device_list, name);
-		}
+		error_text_section("Dump target list", name, dump->device_list);
 		return rc;
 	}
 	if (size == 0) {
@@ -1090,14 +1005,7 @@ check_job_data(struct job_data* job)
 	if (job->target.bootmap_dir != NULL) {
 		rc = misc_check_writable_directory(job->target.bootmap_dir);
 		if (rc) {
-			if (job->name == NULL) {
-				error_text("Target directory '%s'",
-					   job->target.bootmap_dir);
-			} else {
-				error_text("Target directory '%s' in section "
-					   "'%s'",
-					   job->target.bootmap_dir, job->name);
-			}
+			error_text_section("Target directory", job->name, job->target.bootmap_dir);
 			return rc;
 		}
 	}
@@ -1107,10 +1015,11 @@ check_job_data(struct job_data* job)
 		rc = 0;
 		break;
 	case job_ipl:
-		rc = check_job_ipl_data(&job->data.ipl, job->name);
+		rc = check_job_ipl_data(&job->data.ipl, job->name,
+					&job->envblk);
 		break;
 	case job_menu:
-		rc = check_job_menu_data(&job->data.menu);
+		rc = check_job_menu_data(&job->data.menu, &job->envblk);
 		break;
 	case job_segment:
 		rc = check_job_segment_data(&job->data.segment, job->name);
@@ -1233,12 +1142,7 @@ get_parmline(char* filename, char* line, char** parmline, address_t* address,
 		extract_address(filename, &addr);
 		rc = misc_read_file(filename, &buffer, &len, 1);
 		if (rc) {
-			if (section == NULL)
-				error_text("Parmfile '%s'", filename);
-			else {
-				error_text("Parmfile '%s' in section '%s'",
-					   filename, section);
-			}
+			error_text_section("Parmfile", section, filename);
 			free(filename);
 			return rc;
 		}
@@ -1271,17 +1175,6 @@ get_parmline(char* filename, char* line, char** parmline, address_t* address,
 			return -1;
 
 	} else result = NULL;
-	/* Check for maximum length */
-	if (result) {
-		len = strlen(result);
-		if (len > MAXIMUM_PARMLINE_SIZE) {
-			error_text("The length of the parameters line "
-				   "(%d bytes) exceeds the allowed maximum "
-				   "(%d bytes)", len, MAXIMUM_PARMLINE_SIZE);
-			free(result);
-			return -1;
-		}
-	}
 	*parmline = result;
 	*address = addr;
 	return 0;
@@ -1354,30 +1247,33 @@ get_job_from_section_data(char* data[], struct job_data* job, char* section)
 				atol(data[(int) scan_keyword_targetoffset]);
 		/* Fill in name and address of image file */
 
-		job->data.ipl.image = misc_strdup(
+		job->data.ipl.common.image = misc_strdup(
 					data[(int) scan_keyword_image]);
-		if (job->data.ipl.image == NULL)
+		if (job->data.ipl.common.image == NULL)
 			return -1;
-		if (extract_address(job->data.ipl.image,
-				    &job->data.ipl.image_addr)) {
-			job->data.ipl.image_addr = IMAGE_LOAD_ADDRESS;
+		if (extract_address(job->data.ipl.common.image,
+				    &job->data.ipl.common.image_addr)) {
+			job->data.ipl.common.image_addr = IMAGE_LOAD_ADDRESS;
 		}
 		/* Fill in parmline */
 		rc = get_parmline(data[(int) scan_keyword_parmfile],
 				  data[(int) scan_keyword_parameters],
-				  &job->data.ipl.parmline,
-				  &job->data.ipl.parm_addr, section);
+				  &job->data.ipl.common.parmline,
+				  &job->data.ipl.common.parm_addr, section);
 		if (rc)
 			return rc;
+		/* Fill in environment block */
+		job->data.ipl.envblk_addr = UNSPECIFIED_ADDRESS;
+
 		/* Fill in name and address of ramdisk file */
 		if (data[(int) scan_keyword_ramdisk] != NULL) {
-			job->data.ipl.ramdisk =
+			job->data.ipl.common.ramdisk =
 				misc_strdup(data[(int) scan_keyword_ramdisk]);
-			if (job->data.ipl.ramdisk == NULL)
+			if (job->data.ipl.common.ramdisk == NULL)
 				return -1;
-			if (extract_address(job->data.ipl.ramdisk,
-					    &job->data.ipl.ramdisk_addr)) {
-				job->data.ipl.ramdisk_addr =
+			if (extract_address(job->data.ipl.common.ramdisk,
+					    &job->data.ipl.common.ramdisk_addr)) {
+				job->data.ipl.common.ramdisk_addr =
 					UNSPECIFIED_ADDRESS;
 			}
 		}
@@ -1409,30 +1305,30 @@ get_job_from_section_data(char* data[], struct job_data* job, char* section)
 		if (job->data.ipl_tape.device == NULL)
 			return -1;
 		/* Fill in name and address of image file */
-		job->data.ipl_tape.image = misc_strdup(
+		job->data.ipl_tape.common.image = misc_strdup(
 					data[(int) scan_keyword_image]);
-		if (job->data.ipl_tape.image == NULL)
+		if (job->data.ipl_tape.common.image == NULL)
 			return -1;
-		if (extract_address(job->data.ipl_tape.image,
-				    &job->data.ipl_tape.image_addr)) {
-			job->data.ipl_tape.image_addr = IMAGE_LOAD_ADDRESS;
+		if (extract_address(job->data.ipl_tape.common.image,
+				    &job->data.ipl_tape.common.image_addr)) {
+			job->data.ipl_tape.common.image_addr = IMAGE_LOAD_ADDRESS;
 		}
 		/* Fill in parmline */
 		rc = get_parmline(data[(int) scan_keyword_parmfile],
 				  data[(int) scan_keyword_parameters],
-				  &job->data.ipl_tape.parmline,
-				  &job->data.ipl_tape.parm_addr, section);
+				  &job->data.ipl_tape.common.parmline,
+				  &job->data.ipl_tape.common.parm_addr, section);
 		if (rc)
 			return rc;
 		/* Fill in name and address of ramdisk file */
 		if (data[(int) scan_keyword_ramdisk] != NULL) {
-			job->data.ipl_tape.ramdisk =
+			job->data.ipl_tape.common.ramdisk =
 				misc_strdup(data[(int) scan_keyword_ramdisk]);
-			if (job->data.ipl_tape.ramdisk == NULL)
+			if (job->data.ipl_tape.common.ramdisk == NULL)
 				return -1;
-			if (extract_address(job->data.ipl_tape.ramdisk,
-					    &job->data.ipl_tape.ramdisk_addr)) {
-				job->data.ipl_tape.ramdisk_addr =
+			if (extract_address(job->data.ipl_tape.common.ramdisk,
+					    &job->data.ipl_tape.common.ramdisk_addr)) {
+				job->data.ipl_tape.common.ramdisk_addr =
 					UNSPECIFIED_ADDRESS;
 			}
 		}
@@ -1769,15 +1665,15 @@ get_section_job(struct scan_token* scan, char* section, struct job_data* job,
 	if (extra_parmline != NULL) {
 		switch (job->id) {
 		case job_ipl:
-			if (job->data.ipl.parmline == NULL)
+			if (job->data.ipl.common.parmline == NULL)
 				buffer = misc_strdup(extra_parmline);
 			else {
 				buffer = append_parmline(
-						job->data.ipl.parmline,
+						job->data.ipl.common.parmline,
 						extra_parmline);
-				free(job->data.ipl.parmline);
+				free(job->data.ipl.common.parmline);
 			}
-			job->data.ipl.parmline = buffer;
+			job->data.ipl.common.parmline = buffer;
 			if (buffer == NULL)
 				return -1;
 			break;
@@ -1895,6 +1791,45 @@ get_job_from_config_file(struct command_line* cmdline, struct job_data* job)
 	return rc;
 }
 
+static int get_job_envblk_data(struct job_data *job, char *import_hint)
+{
+	struct job_envblk_data *data = &job->envblk;
+	int fd;
+
+	switch (job->id) {
+	case job_ipl:
+	case job_menu:
+		break;
+	default:
+		return 0;
+	}
+	fd = open(job->target.bootmap_dir, O_RDONLY);
+	if (fd < 0) {
+		error_reason(strerror(errno));
+		error_text("Could not open bootmap dir");
+		return -1;
+	}
+	if (envblk_size_get(fd, &data->size)) {
+		close(fd);
+		error_text("Could not get environment block size");
+		return -1;
+	}
+	close(fd);
+	data->buf = misc_malloc(data->size);
+	if (data->buf == NULL) {
+		error_text("Could not allocate environment block");
+		return -1;
+	}
+	envblk_create_blank(data->buf, data->size);
+
+	if (envblk_import(import_hint ?: ENVBLK_DEFAULT_IMPORT_SOURCE,
+			  data->buf, data->size)) {
+		free(data->buf);
+		data->buf = NULL;
+		return -1;
+	}
+	return 0;
+}
 
 int
 job_get(int argc, char* argv[], struct job_data** data)
@@ -1917,6 +1852,9 @@ job_get(int argc, char* argv[], struct job_data** data)
 	job->data.mvdump.force = cmdline.force;
 	job->dry_run = cmdline.dry_run;
 	job->is_secure =  SECURE_BOOT_UNDEFINED;
+	if (job->verbose)
+		printf("Looking for components in '%s'\n", util_libdir());
+
 	/* Get job data from user input */
 	if (cmdline.help) {
 		job->command_line = 1;
@@ -1940,6 +1878,11 @@ job_get(int argc, char* argv[], struct job_data** data)
 	else if (job->id != job_menu && job->is_secure == SECURE_BOOT_UNDEFINED)
 		job->is_secure = SECURE_BOOT_AUTO;
 
+	rc = get_job_envblk_data(job, cmdline.envblk_import_hint);
+	if (rc) {
+		job_free(job);
+		return -1;
+	}
 	/* Check job data for validity */
 	rc = check_job_data(job);
 	if (rc) {
